# Generated by roxygen2: do not edit by hand

S3method("[",draws_array)
S3method("[",draws_df)
S3method("[",draws_list)
S3method("[",draws_matrix)
S3method("[",draws_rvars)
S3method("[",rvar)
S3method("[<-",rvar)
S3method("[[",rvar)
S3method("[[<-",rvar)
S3method("dim<-",rvar)
S3method("dimnames<-",rvar)
S3method("levels<-",rvar)
S3method("names<-",rvar)
S3method("variables<-",draws_array)
S3method("variables<-",draws_df)
S3method("variables<-",draws_list)
S3method("variables<-",draws_matrix)
S3method("variables<-",draws_rvars)
S3method(.subset_draws,draws_array)
S3method(.subset_draws,draws_df)
S3method(.subset_draws,draws_list)
S3method(.subset_draws,draws_matrix)
S3method(.subset_draws,draws_rvars)
S3method(Math,rvar)
S3method(Math,rvar_factor)
S3method(Ops,rvar)
S3method(Ops,rvar_factor)
S3method(Ops,rvar_ordered)
S3method(Pr,default)
S3method(Pr,logical)
S3method(Pr,rvar)
S3method(Summary,rvar)
S3method(all,rvar)
S3method(all.equal,rvar)
S3method(any,rvar)
S3method(anyDuplicated,rvar)
S3method(anyDuplicated,rvar_factor)
S3method(anyNA,rvar)
S3method(aperm,rvar)
S3method(as.data.frame,rvar)
S3method(as.list,rvar)
S3method(as.vector,rvar)
S3method(as_draws,default)
S3method(as_draws,draws)
S3method(as_draws,rvar)
S3method(as_draws_array,default)
S3method(as_draws_array,draws_array)
S3method(as_draws_array,draws_df)
S3method(as_draws_array,draws_list)
S3method(as_draws_array,draws_matrix)
S3method(as_draws_array,draws_rvars)
S3method(as_draws_array,mcmc)
S3method(as_draws_array,mcmc.list)
S3method(as_draws_df,data.frame)
S3method(as_draws_df,default)
S3method(as_draws_df,draws_array)
S3method(as_draws_df,draws_df)
S3method(as_draws_df,draws_list)
S3method(as_draws_df,draws_matrix)
S3method(as_draws_df,draws_rvars)
S3method(as_draws_df,mcmc)
S3method(as_draws_df,mcmc.list)
S3method(as_draws_list,default)
S3method(as_draws_list,draws_array)
S3method(as_draws_list,draws_df)
S3method(as_draws_list,draws_list)
S3method(as_draws_list,draws_matrix)
S3method(as_draws_list,draws_rvars)
S3method(as_draws_list,mcmc)
S3method(as_draws_list,mcmc.list)
S3method(as_draws_matrix,default)
S3method(as_draws_matrix,draws_array)
S3method(as_draws_matrix,draws_df)
S3method(as_draws_matrix,draws_list)
S3method(as_draws_matrix,draws_matrix)
S3method(as_draws_matrix,draws_rvars)
S3method(as_draws_matrix,mcmc)
S3method(as_draws_matrix,mcmc.list)
S3method(as_draws_rvars,default)
S3method(as_draws_rvars,draws_array)
S3method(as_draws_rvars,draws_df)
S3method(as_draws_rvars,draws_list)
S3method(as_draws_rvars,draws_matrix)
S3method(as_draws_rvars,draws_rvars)
S3method(as_draws_rvars,list)
S3method(as_draws_rvars,mcmc)
S3method(as_draws_rvars,mcmc.list)
S3method(as_draws_rvars,rvar)
S3method(as_tibble,rvar)
S3method(bind_draws,"NULL")
S3method(bind_draws,draws_array)
S3method(bind_draws,draws_df)
S3method(bind_draws,draws_list)
S3method(bind_draws,draws_matrix)
S3method(bind_draws,draws_rvars)
S3method(bind_draws,list)
S3method(broadcast_and_bind_rvars,rvar)
S3method(broadcast_and_bind_rvars,rvar_factor)
S3method(c,rvar)
S3method(cbind,rvar)
S3method(cdf,rvar)
S3method(cdf,rvar_factor)
S3method(cdf,rvar_ordered)
S3method(chain_ids,"NULL")
S3method(chain_ids,draws_array)
S3method(chain_ids,draws_df)
S3method(chain_ids,draws_list)
S3method(chain_ids,draws_matrix)
S3method(chain_ids,draws_rvars)
S3method(chol,rvar)
S3method(density,rvar)
S3method(density,rvar_factor)
S3method(dim,rvar)
S3method(dimnames,rvar)
S3method(dissent,default)
S3method(dissent,rvar)
S3method(draw_ids,"NULL")
S3method(draw_ids,draws_array)
S3method(draw_ids,draws_df)
S3method(draw_ids,draws_list)
S3method(draw_ids,draws_matrix)
S3method(draw_ids,draws_rvars)
S3method(draw_ids,rvar)
S3method(duplicated,rvar)
S3method(duplicated,rvar_factor)
S3method(entropy,default)
S3method(entropy,rvar)
S3method(ess_basic,default)
S3method(ess_basic,rvar)
S3method(ess_bulk,default)
S3method(ess_bulk,rvar)
S3method(ess_mean,default)
S3method(ess_mean,rvar)
S3method(ess_quantile,default)
S3method(ess_quantile,rvar)
S3method(ess_sd,default)
S3method(ess_sd,rvar)
S3method(ess_tail,default)
S3method(ess_tail,rvar)
S3method(extract_list_of_variable_arrays,default)
S3method(extract_list_of_variable_arrays,draws)
S3method(extract_variable,default)
S3method(extract_variable,draws)
S3method(extract_variable,draws_df)
S3method(extract_variable,draws_list)
S3method(extract_variable,draws_rvars)
S3method(extract_variable_array,default)
S3method(extract_variable_array,draws)
S3method(extract_variable_matrix,default)
S3method(extract_variable_matrix,draws)
S3method(extract_variable_matrix,draws_df)
S3method(extract_variable_matrix,draws_list)
S3method(extract_variable_matrix,draws_rvars)
S3method(format,rvar)
S3method(format_glimpse,rvar)
S3method(get_rvar_class,default)
S3method(get_rvar_class,factor)
S3method(get_rvar_class,ordered)
S3method(is.array,rvar)
S3method(is.finite,rvar)
S3method(is.infinite,rvar)
S3method(is.matrix,rvar)
S3method(is.na,rvar)
S3method(is.nan,rvar)
S3method(iteration_ids,"NULL")
S3method(iteration_ids,draws_array)
S3method(iteration_ids,draws_df)
S3method(iteration_ids,draws_list)
S3method(iteration_ids,draws_matrix)
S3method(iteration_ids,draws_rvars)
S3method(iteration_ids,rvar)
S3method(length,rvar)
S3method(levels,rvar)
S3method(mad,default)
S3method(mad,rvar)
S3method(mad,rvar_ordered)
S3method(match,default)
S3method(match,rvar)
S3method(max,rvar)
S3method(mcse_mean,default)
S3method(mcse_mean,rvar)
S3method(mcse_quantile,default)
S3method(mcse_quantile,rvar)
S3method(mcse_sd,default)
S3method(mcse_sd,rvar)
S3method(mean,rvar)
S3method(median,rvar)
S3method(merge_chains,draws_array)
S3method(merge_chains,draws_df)
S3method(merge_chains,draws_list)
S3method(merge_chains,draws_matrix)
S3method(merge_chains,draws_rvars)
S3method(merge_chains,rvar)
S3method(min,rvar)
S3method(modal_category,default)
S3method(modal_category,rvar)
S3method(mutate_variables,draws_array)
S3method(mutate_variables,draws_df)
S3method(mutate_variables,draws_list)
S3method(mutate_variables,draws_matrix)
S3method(mutate_variables,draws_rvars)
S3method(names,rvar)
S3method(nchains,"NULL")
S3method(nchains,draws_array)
S3method(nchains,draws_df)
S3method(nchains,draws_list)
S3method(nchains,draws_matrix)
S3method(nchains,draws_rvars)
S3method(nchains,rvar)
S3method(ndraws,"NULL")
S3method(ndraws,draws_array)
S3method(ndraws,draws_df)
S3method(ndraws,draws_list)
S3method(ndraws,draws_matrix)
S3method(ndraws,draws_rvars)
S3method(ndraws,rvar)
S3method(niterations,"NULL")
S3method(niterations,draws_array)
S3method(niterations,draws_df)
S3method(niterations,draws_list)
S3method(niterations,draws_matrix)
S3method(niterations,draws_rvars)
S3method(niterations,rvar)
S3method(nvariables,"NULL")
S3method(nvariables,draws)
S3method(order_draws,draws_array)
S3method(order_draws,draws_df)
S3method(order_draws,draws_list)
S3method(order_draws,draws_matrix)
S3method(order_draws,draws_rvars)
S3method(order_draws,rvar)
S3method(pareto_convergence_rate,default)
S3method(pareto_convergence_rate,rvar)
S3method(pareto_diags,default)
S3method(pareto_diags,rvar)
S3method(pareto_khat,default)
S3method(pareto_khat,rvar)
S3method(pareto_khat_threshold,default)
S3method(pareto_khat_threshold,rvar)
S3method(pareto_min_ss,default)
S3method(pareto_min_ss,rvar)
S3method(pareto_pit,default)
S3method(pareto_pit,draws_matrix)
S3method(pareto_pit,rvar)
S3method(pareto_smooth,default)
S3method(pareto_smooth,rvar)
S3method(pillar_shaft,rvar)
S3method(pit,default)
S3method(pit,draws_matrix)
S3method(pit,rvar)
S3method(print,draws_array)
S3method(print,draws_df)
S3method(print,draws_list)
S3method(print,draws_matrix)
S3method(print,draws_rvars)
S3method(print,draws_summary)
S3method(print,rvar)
S3method(prod,rvar)
S3method(quantile,rvar)
S3method(quantile,rvar_factor)
S3method(quantile,rvar_ordered)
S3method(quantile2,default)
S3method(quantile2,rvar)
S3method(range,rvar)
S3method(rbind,rvar)
S3method(remove_variables,draws_array)
S3method(remove_variables,draws_df)
S3method(remove_variables,draws_list)
S3method(remove_variables,draws_matrix)
S3method(remove_variables,draws_rvars)
S3method(remove_variables,list)
S3method(rename_variables,draws)
S3method(rep,rvar)
S3method(rep.int,rvar,rep_int_rvar)
S3method(rep_len,rvar)
S3method(repair_draws,draws_array)
S3method(repair_draws,draws_df)
S3method(repair_draws,draws_list)
S3method(repair_draws,draws_matrix)
S3method(repair_draws,draws_rvars)
S3method(repair_draws,rvar)
S3method(resample_draws,draws)
S3method(resample_draws,rvar)
S3method(reserved_variables,default)
S3method(reserved_variables,draws_array)
S3method(reserved_variables,draws_df)
S3method(reserved_variables,draws_list)
S3method(reserved_variables,draws_matrix)
S3method(reserved_variables,draws_rvars)
S3method(rhat,default)
S3method(rhat,rvar)
S3method(rhat_basic,default)
S3method(rhat_basic,rvar)
S3method(rhat_nested,default)
S3method(rhat_nested,rvar)
S3method(sd,default)
S3method(sd,rvar)
S3method(split_chains,draws)
S3method(str,rvar)
S3method(subset,draws)
S3method(subset_draws,draws_array)
S3method(subset_draws,draws_df)
S3method(subset_draws,draws_list)
S3method(subset_draws,draws_matrix)
S3method(subset_draws,draws_rvars)
S3method(subset_draws,rvar)
S3method(sum,rvar)
S3method(summarise_draws,default)
S3method(summarise_draws,draws)
S3method(summarise_draws,rvar)
S3method(summary,draws)
S3method(summary,rvar)
S3method(t,rvar)
S3method(thin_draws,draws)
S3method(thin_draws,rvar)
S3method(unique,rvar)
S3method(unique,rvar_factor)
S3method(var,default)
S3method(var,rvar)
S3method(variables,"NULL")
S3method(variables,draws_array)
S3method(variables,draws_df)
S3method(variables,draws_list)
S3method(variables,draws_matrix)
S3method(variables,draws_rvars)
S3method(variance,draws_array)
S3method(variance,draws_matrix)
S3method(variance,rvar)
S3method(vec_cast,character.rvar)
S3method(vec_cast,character.rvar_factor)
S3method(vec_cast,character.rvar_ordered)
S3method(vec_cast,distribution.rvar)
S3method(vec_cast,rvar.character)
S3method(vec_cast,rvar.distribution)
S3method(vec_cast,rvar.double)
S3method(vec_cast,rvar.factor)
S3method(vec_cast,rvar.integer)
S3method(vec_cast,rvar.logical)
S3method(vec_cast,rvar.ordered)
S3method(vec_cast,rvar.rvar)
S3method(vec_cast,rvar.rvar_factor)
S3method(vec_cast,rvar.rvar_ordered)
S3method(vec_cast,rvar_factor.character)
S3method(vec_cast,rvar_factor.double)
S3method(vec_cast,rvar_factor.factor)
S3method(vec_cast,rvar_factor.integer)
S3method(vec_cast,rvar_factor.logical)
S3method(vec_cast,rvar_factor.ordered)
S3method(vec_cast,rvar_factor.rvar)
S3method(vec_cast,rvar_factor.rvar_factor)
S3method(vec_cast,rvar_factor.rvar_ordered)
S3method(vec_cast,rvar_ordered.character)
S3method(vec_cast,rvar_ordered.double)
S3method(vec_cast,rvar_ordered.factor)
S3method(vec_cast,rvar_ordered.integer)
S3method(vec_cast,rvar_ordered.logical)
S3method(vec_cast,rvar_ordered.ordered)
S3method(vec_cast,rvar_ordered.rvar)
S3method(vec_cast,rvar_ordered.rvar_factor)
S3method(vec_cast,rvar_ordered.rvar_ordered)
S3method(vec_proxy,rvar)
S3method(vec_proxy_compare,rvar)
S3method(vec_proxy_equal,rvar)
S3method(vec_proxy_order,rvar)
S3method(vec_ptype,rvar)
S3method(vec_ptype,rvar_factor)
S3method(vec_ptype,rvar_ordered)
S3method(vec_ptype2,character.rvar_factor)
S3method(vec_ptype2,character.rvar_ordered)
S3method(vec_ptype2,distribution.rvar)
S3method(vec_ptype2,double.rvar)
S3method(vec_ptype2,factor.rvar_factor)
S3method(vec_ptype2,factor.rvar_ordered)
S3method(vec_ptype2,integer.rvar)
S3method(vec_ptype2,logical.rvar)
S3method(vec_ptype2,ordered.rvar_factor)
S3method(vec_ptype2,ordered.rvar_ordered)
S3method(vec_ptype2,rvar.distribution)
S3method(vec_ptype2,rvar.double)
S3method(vec_ptype2,rvar.integer)
S3method(vec_ptype2,rvar.logical)
S3method(vec_ptype2,rvar.rvar)
S3method(vec_ptype2,rvar_factor.character)
S3method(vec_ptype2,rvar_factor.factor)
S3method(vec_ptype2,rvar_factor.ordered)
S3method(vec_ptype2,rvar_factor.rvar_factor)
S3method(vec_ptype2,rvar_ordered.character)
S3method(vec_ptype2,rvar_ordered.factor)
S3method(vec_ptype2,rvar_ordered.ordered)
S3method(vec_ptype2,rvar_ordered.rvar_ordered)
S3method(vec_ptype_abbr,rvar)
S3method(vec_ptype_abbr,rvar_factor)
S3method(vec_ptype_abbr,rvar_ordered)
S3method(vec_ptype_full,rvar)
S3method(vec_restore,rvar)
S3method(vec_restore,rvar_factor)
S3method(vec_restore,rvar_ordered)
S3method(weight_draws,draws_array)
S3method(weight_draws,draws_df)
S3method(weight_draws,draws_list)
S3method(weight_draws,draws_matrix)
S3method(weight_draws,draws_rvars)
S3method(weights,draws)
export("%**%")
export("%in%")
export("draws_of<-")
export("variables<-")
export(E)
export(Pr)
export(as_draws)
export(as_draws_array)
export(as_draws_df)
export(as_draws_list)
export(as_draws_matrix)
export(as_draws_rvars)
export(as_rvar)
export(as_rvar_factor)
export(as_rvar_integer)
export(as_rvar_logical)
export(as_rvar_numeric)
export(as_rvar_ordered)
export(autocorrelation)
export(autocovariance)
export(bind_draws)
export(cdf)
export(chain_ids)
export(default_convergence_measures)
export(default_mcse_measures)
export(default_summary_measures)
export(diag)
export(dissent)
export(draw_ids)
export(draws_array)
export(draws_df)
export(draws_list)
export(draws_matrix)
export(draws_of)
export(draws_rvars)
export(drop)
export(entropy)
export(ess_basic)
export(ess_bulk)
export(ess_mean)
export(ess_median)
export(ess_quantile)
export(ess_sd)
export(ess_tail)
export(example_draws)
export(extract_list_of_variable_arrays)
export(extract_variable)
export(extract_variable_array)
export(extract_variable_matrix)
export(for_each_draw)
export(gpdfit)
export(is_constant)
export(is_draws)
export(is_draws_array)
export(is_draws_df)
export(is_draws_list)
export(is_draws_matrix)
export(is_draws_rvars)
export(is_rvar)
export(is_rvar_factor)
export(is_rvar_ordered)
export(iteration_ids)
export(mad)
export(match)
export(mcse_mean)
export(mcse_median)
export(mcse_quantile)
export(mcse_sd)
export(merge_chains)
export(modal_category)
export(mutate_variables)
export(nchains)
export(ndraws)
export(niterations)
export(nvariables)
export(order_draws)
export(pareto_convergence_rate)
export(pareto_diags)
export(pareto_khat)
export(pareto_khat_threshold)
export(pareto_min_ss)
export(pareto_pit)
export(pareto_smooth)
export(pgeneralized_pareto)
export(pit)
export(ps_convergence_rate)
export(ps_khat_threshold)
export(ps_min_ss)
export(ps_tail)
export(ps_tail_length)
export(qgeneralized_pareto)
export(quantile2)
export(r_scale)
export(rdo)
export(rename_variables)
export(repair_draws)
export(resample_draws)
export(reserved_variables)
export(rfun)
export(rhat)
export(rhat_basic)
export(rhat_nested)
export(rstar)
export(rvar)
export(rvar_all)
export(rvar_any)
export(rvar_apply)
export(rvar_factor)
export(rvar_ifelse)
export(rvar_is_finite)
export(rvar_is_infinite)
export(rvar_is_na)
export(rvar_is_nan)
export(rvar_mad)
export(rvar_max)
export(rvar_mean)
export(rvar_median)
export(rvar_min)
export(rvar_ordered)
export(rvar_prod)
export(rvar_quantile)
export(rvar_range)
export(rvar_rng)
export(rvar_sd)
export(rvar_sum)
export(rvar_var)
export(sd)
export(set_variables)
export(split_chains)
export(subset_draws)
export(summarise_draws)
export(summarize_draws)
export(thin_draws)
export(u_scale)
export(uniformity_test)
export(var)
export(variables)
export(variance)
export(weight_draws)
export(z_scale)
exportMethods(diag)
exportMethods(drop)
import(checkmate)
import(stats)
importFrom(abind,abind)
importFrom(distributional,cdf)
importFrom(distributional,variance)
importFrom(methods,callNextMethod)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,setOldClass)
importFrom(pillar,format_glimpse)
importFrom(pillar,new_pillar_shaft_simple)
importFrom(pillar,pillar_shaft)
importFrom(rlang,as_label)
importFrom(rlang,as_name)
importFrom(rlang,caller_env)
importFrom(rlang,dots_list)
importFrom(rlang,enquo)
importFrom(rlang,enquos)
importFrom(rlang,eval_tidy)
importFrom(rlang,expr)
importFrom(rlang,inject)
importFrom(rlang,is_missing)
importFrom(rlang,missing_arg)
importFrom(rlang,quo_get_env)
importFrom(rlang,quo_get_expr)
importFrom(rlang,quos)
importFrom(tensorA,as.tensor)
importFrom(tensorA,chol.tensor)
importFrom(tensorA,mul.tensor)
importFrom(tibble,as_tibble)
importFrom(utils,lsf.str)
importFrom(utils,str)
importFrom(vctrs,new_vctr)
importFrom(vctrs,vec_cast)
importFrom(vctrs,vec_cast_common)
importFrom(vctrs,vec_proxy)
importFrom(vctrs,vec_proxy_compare)
importFrom(vctrs,vec_proxy_equal)
importFrom(vctrs,vec_proxy_order)
importFrom(vctrs,vec_ptype)
importFrom(vctrs,vec_ptype2)
importFrom(vctrs,vec_ptype_abbr)
importFrom(vctrs,vec_ptype_full)
importFrom(vctrs,vec_restore)
importFrom(vctrs,vec_slice)
