% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gpd.R
\name{gpdfit}
\alias{gpdfit}
\title{Estimate parameters of the Generalized Pareto distribution}
\usage{
gpdfit(x, wip = TRUE, min_grid_pts = 30, sort_x = TRUE, weights = NULL)
}
\arguments{
\item{x}{A numeric vector. The sample from which to estimate the
parameters.}

\item{wip}{Logical indicating whether to adjust \eqn{k} based on a
weakly informative Gaussian prior centered on 0.5. Defaults to
\code{TRUE}.}

\item{min_grid_pts}{The minimum number of grid points used in the
fitting algorithm. The actual number used is \code{min_grid_pts + floor(sqrt(length(x)))}.}

\item{sort_x}{If \code{TRUE} (the default), the first step in the
fitting algorithm is to sort the elements of \code{x}. If \code{x} is
already sorted in ascending order then \code{sort_x} can be set to
\code{FALSE} to skip the initial sorting step.}

\item{weights}{An optional numeric vector of positive weights the same
length as \code{x}. If \code{NULL} (the default), all observations are
weighted equally and the result is identical to the unweighted fit.
Weights are normalized internally to sum to \code{length(x)}.}
}
\value{
A named list with components \code{k} and \code{sigma}.
}
\description{
Given a sample \eqn{x}, Estimate the parameters \eqn{k} and
\eqn{\sigma} of the generalized Pareto distribution (GPD), assuming
the location parameter is 0. By default the fit uses a prior for
\eqn{k} (this is in addition to the prior described by Zhang and
Stephens, 2009), which will stabilize estimates for very small
sample sizes (and low effective sample sizes in the case of MCMC
samples). The weakly informative prior is a Gaussian prior centered
at 0.5 (see details in Vehtari et al., 2024). This is used
internally but is exported for use by other packages.
}
\details{
Here the parameter \eqn{k} is the negative of \eqn{k} in Zhang &
Stephens (2009).
}
\references{
Zhang, J., and Stephens, M. A. (2009). A new and efficient estimation method
for the generalized Pareto distribution. \emph{Technometrics} \strong{51}, 316-325.
}
\seealso{
Other helper-functions: 
\code{\link{ps_convergence_rate}()},
\code{\link{ps_khat_threshold}()},
\code{\link{ps_min_ss}()},
\code{\link{ps_tail_length}()}
}
\concept{helper-functions}
\keyword{internal}
